mod config;
mod weather;

use chrono::{TimeZone, Utc};
use chrono_tz::Tz;
use config::Config;
use weather::{fetch_weather, WeatherSummary};

#[tokio::main]
async fn main() {
    let config = match Config::load() {
        Ok(cfg) => cfg,
        Err(e) => {
            eprintln!("Failed to load config: {e}");
            std::process::exit(1);
        }
    };

    let weather: Option<WeatherSummary> =
        match fetch_weather(&config.location, &config.weather_api_key).await {
            Ok(data) => Some(data),
            Err(e) => {
                eprintln!("Failed to fetch weather: {e}");
                None
            }
        };
    if let Some(w) = weather {
        if w.obs_time_unix > 0 {
            match config.timezone.parse::<Tz>() {
                Ok(tz) => {
                    let dt_local = tz.from_utc_datetime(
                        &Utc.timestamp_opt(w.obs_time_unix, 0).unwrap().naive_utc(),
                    );
                    println!(
                        "Weather (as of {}):",
                        dt_local.format("%Y-%m-%d %H:%M:%S %Z")
                    );
                }
                Err(_) => {
                    println!(
                        "Weather (as of unix timestamp {}) (timezone parse error)",
                        w.obs_time_unix
                    );
                }
            }
        } else {
            println!("Weather:");
        }
        println!(" - Temperature: {}", w.temp);
        println!(" - Conditions: {}", w.current_desc);
        println!(" - High: {}, Low: {}", w.high, w.low);
        println!(" - Forecast: {}", w.daily_desc);
    } else {
        println!("Weather: N/A");
    }

    let pollen = get_pollen_count(&config);
    println!("\nPollen count: {pollen}");

    println!();
    println!("Upcoming calendar events:");
    let events = get_calendar_events(&config, config.calendar_event_count);
    for (i, event) in events.iter().enumerate() {
        println!("{}. {}", i + 1, event);
    }

    println!();
    println!("Inbound Shopify packages:");
    let packages = get_shopify_packages(&config);
    for (i, pkg) in packages.iter().enumerate() {
        println!("{}. {}", i + 1, pkg);
    }
}

// Placeholder stubs, replace with actual API calls later
fn get_pollen_count(_config: &Config) -> String {
    // TODO: Call pollen count API
    "Medium (Tree: 5, Grass: 2, Weed: 1)".to_string()
}

fn get_calendar_events(_config: &Config, n: usize) -> Vec<String> {
    // TODO: Call calendar API
    vec![
        "Team meeting at 10:00 AM".to_string(),
        "Lunch with Alex at 12:30 PM".to_string(),
        "Dentist appointment at 3:00 PM".to_string(),
    ]
    .into_iter()
    .take(n)
    .collect()
}

fn get_shopify_packages(_config: &Config) -> Vec<String> {
    // TODO: Call Shopify API
    vec![
        "Order #1234: Shipped - Arriving May 13".to_string(),
        "Order #5678: In transit - Arriving May 15".to_string(),
    ]
}
